precision lowp float;

uniform samplerCube tex3;
uniform sampler2D tex1;
uniform sampler2D tex2;
uniform sampler2D tex4;
uniform sampler2D tex5;

varying vec2 v_v2TexCoord;
varying vec3 v_v3WorldPosition;

uniform mat3 u_m33NormalTransform;
uniform vec4 u_v4Color;
uniform vec3 u_v3LightDir;
uniform vec3 u_v3EyePosition;
uniform float u_fBrightness;

void main() 
{	
	vec4 v4TextureNormal = texture2D(tex1, v_v2TexCoord); // v4TextureNormal.w contains gloss
	vec4 v4TextureDiffuse = texture2D(tex2, v_v2TexCoord); // v4TextureDiffuse.w contains ambient occlusion
	vec4 v4TexTemp;
		
	// todo: Is there a way to pre transform to avoid multiplying by u_m33NormalTransform during the fragment shader?
	// todo: I think it might require the cube map to be rotated, which wont work for different car body parts.
	// todo: It is an expensive per pixel operation so it would be nice to avoid it.
	vec3 v3Incident = v_v3WorldPosition - u_v3EyePosition;
	vec3 v3Normal = (2.0 * (v4TextureNormal.xyz - vec3(0.5, 0.5, 0.5))) * u_m33NormalTransform;
	v3Incident = reflect(v3Incident, v3Normal);
	
	vec4 v4TextureCube = textureCube(tex3, v3Incident);
	
	gl_FragColor.xyz = 
		v4TextureCube.xyz * v4TextureNormal.w * v4TextureDiffuse.w * 1.5 + 
		v4TextureDiffuse.xyz * (dot(v3Normal, u_v3LightDir) + 1.5) * u_v4Color.xyz * v4TextureDiffuse.w;
				
	vec3 v3Tmp = 1.0 - gl_FragColor.rgb;
	gl_FragColor.rgb = gl_FragColor.rgb * (2.0 - u_fBrightness) + (1.0 - v3Tmp * v3Tmp * v3Tmp) * (u_fBrightness - 1.0);
	
	gl_FragColor.w = 0.25;
}
