precision lowp float;

uniform lowp samplerCube tex3;
uniform lowp sampler2D tex1;
uniform lowp sampler2D tex2;
uniform lowp sampler2D tex4;
uniform lowp sampler2D tex5;

varying mediump vec2 v_v2TexCoord;
varying mediump vec3 v_v3WorldPosition;

uniform mediump mat3 u_m33NormalTransform;
uniform mediump vec4 u_v4Color;
uniform mediump vec3 u_v3LightDir;
uniform mediump vec3 u_v3EyePosition;

void main() 
{	
	lowp vec4 v4TextureNormal = texture2D(tex1, v_v2TexCoord); // v4TextureNormal.w contains gloss
	lowp vec4 v4TextureDiffuse = texture2D(tex2, v_v2TexCoord); // v4TextureDiffuse.w contains ambient occlusion
	
	// todo: Is there a way to pre transform to avoid multiplying by u_m33NormalTransform during the fragment shader?
	// todo: I think it might require the cube map to be rotated, which wont work for different car body parts.
	// todo: It is an expensive per pixel operation so it would be nice to avoid it.
	mediump vec3 v3Incident = v_v3WorldPosition - u_v3EyePosition;
	lowp vec3 v3Normal = (2.0 * (v4TextureNormal.xyz - vec3(0.5, 0.5, 0.5))) * u_m33NormalTransform;
	v3Incident = reflect(v3Incident, v3Normal);
	
	lowp vec4 v4TextureCube = textureCube(tex3, v3Incident);
	
	gl_FragColor.xyz = 
		v4TextureCube.xyz * v4TextureNormal.w * v4TextureDiffuse.w * 1.5 + 
		v4TextureDiffuse.xyz * (dot(v3Normal, u_v3LightDir) + 1.5) * u_v4Color.xyz * v4TextureDiffuse.w;
				
	gl_FragColor.w = 0.25;
}
